var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  default: () => dtsPlugin,
  dtsPlugin: () => dtsPlugin,
  getTSConfig: () => getTSConfig,
  humanizeFileSize: () => humanizeFileSize
});
module.exports = __toCommonJS(src_exports);

// src/lib/getTSConfig.ts
var import_fs = require("fs");
var import_lodash = __toESM(require("lodash.merge"));
var import_path = require("path");
var import_typescript = __toESM(require("typescript"));
function getTSConfig(opts) {
  var _a, _b;
  let configPath = (_b = opts.configPath) != null ? _b : import_typescript.default.findConfigFile(
    (_a = opts.searchPath) != null ? _a : process.cwd(),
    import_typescript.default.sys.fileExists,
    opts.configName
  );
  if (!configPath) {
    throw new Error("No config file found");
  }
  if (configPath.startsWith(".")) {
    configPath = require.resolve(configPath);
  }
  const config = import_typescript.default.readConfigFile(
    configPath,
    (path) => (0, import_fs.readFileSync)(path, "utf-8")
  );
  if (config.config.extends) {
    const parentConfig = getTSConfig({
      ...opts,
      configPath: (0, import_path.resolve)((0, import_path.dirname)(configPath), config.config.extends)
    });
    config.config = (0, import_lodash.default)(config.config, parentConfig);
  }
  if (config.error) {
    throw config.error;
  } else {
    return config.config;
  }
}

// src/lib/humanizeFileSize.ts
function humanizeFileSize(size) {
  const i = Math.floor(Math.log(size) / Math.log(1024));
  return Math.round(size / Math.pow(1024, i) * 100) / 100 + ["b", "kb", "mb", "gb", "tb"][i];
}

// src/plugin.ts
var import_chalk = __toESM(require("chalk"));
var import_fs2 = require("fs");
var import_path3 = require("path");
var import_typescript3 = __toESM(require("typescript"));

// src/lib/getCompilerOptions.ts
var import_crypto = require("crypto");
var import_path2 = require("path");
var import_tmp = require("tmp");
var import_typescript2 = __toESM(require("typescript"));
function getCompilerOptions(opts) {
  var _a, _b, _c, _d;
  const compilerOptions = import_typescript2.default.convertCompilerOptionsFromJson(
    opts.tsconfig.compilerOptions,
    process.cwd()
  ).options;
  compilerOptions.declaration = true;
  compilerOptions.emitDeclarationOnly = true;
  if (!compilerOptions.declarationDir) {
    compilerOptions.declarationDir = (_b = (_a = compilerOptions.declarationDir) != null ? _a : opts.esbuildOptions.outdir) != null ? _b : compilerOptions.outDir;
  }
  if (compilerOptions.incremental && !compilerOptions.tsBuildInfoFile) {
    const configHash = (0, import_crypto.createHash)("sha256").update(
      JSON.stringify({
        compilerOptions,
        __buildContext: (_c = opts.pluginOptions) == null ? void 0 : _c.__buildContext
      })
    ).digest("hex");
    compilerOptions.tsBuildInfoFile = (0, import_path2.resolve)(
      (_d = opts.pluginOptions.buildInfoDir) != null ? _d : import_tmp.tmpdir,
      `esbuild-plugin-dts-${configHash}.tsbuildinfo`
    );
  }
  compilerOptions.listEmittedFiles = true;
  return compilerOptions;
}

// src/lib/logger.ts
function createLogger(logLevel) {
  const levels = [
    "verbose",
    "debug",
    "info",
    "warning",
    "error",
    "silent"
  ];
  for (const l of levels) {
    if (l === logLevel) {
      break;
    } else {
      levels.splice(levels.indexOf(l), 1);
    }
  }
  return {
    info: (...msg) => {
      if (levels.includes("info"))
        console.log(...msg);
    }
  };
}

// src/plugin.ts
var dtsPlugin = (opts = {}) => ({
  name: "dts-plugin",
  async setup(build) {
    const log = createLogger(build.initialOptions.logLevel);
    const config = opts.tsconfig && typeof opts.tsconfig !== "string" ? opts.tsconfig : getTSConfig({
      configPath: opts.tsconfig
    });
    const compilerOptions = getCompilerOptions({
      tsconfig: config,
      pluginOptions: opts,
      esbuildOptions: build.initialOptions
    });
    const compilerHost = compilerOptions.incremental ? import_typescript3.default.createIncrementalCompilerHost(compilerOptions) : import_typescript3.default.createCompilerHost(compilerOptions);
    const inputFiles = [];
    build.onLoad({ filter: /(\.tsx|\.ts)$/ }, async (args) => {
      var _a;
      inputFiles.push(args.path);
      compilerHost.getSourceFile(
        args.path,
        (_a = compilerOptions.target) != null ? _a : import_typescript3.default.ScriptTarget.Latest,
        (m) => console.log(m),
        true
      );
      return {};
    });
    build.onEnd(() => {
      let compilerProgram;
      if (compilerOptions.incremental) {
        compilerProgram = import_typescript3.default.createIncrementalProgram({
          options: compilerOptions,
          host: compilerHost,
          rootNames: inputFiles
        });
      } else {
        compilerProgram = import_typescript3.default.createProgram(
          inputFiles,
          compilerOptions,
          compilerHost
        );
      }
      const startTime = Date.now();
      const emitResult = compilerProgram.emit();
      if (emitResult.emitSkipped || typeof emitResult.emittedFiles === "undefined") {
        log.info(import_chalk.default`{yellow No declarations emitted}`);
      } else {
        for (const emittedFile of emitResult.emittedFiles) {
          const emittedPath = (0, import_path3.resolve)(emittedFile);
          if ((0, import_fs2.existsSync)(emittedPath) && emittedPath !== compilerOptions.tsBuildInfoFile) {
            const stat = (0, import_fs2.lstatSync)(emittedPath);
            const pathFromContentRoot = emittedPath.replace((0, import_path3.resolve)(process.cwd()), "").replace(/^[\\/]/, "");
            const humanFileSize = humanizeFileSize(stat.size);
            log.info(
              import_chalk.default`  {bold ${pathFromContentRoot}} {cyan ${humanFileSize}}`
            );
          }
        }
      }
      log.info(
        import_chalk.default`{green Finished compiling declarations in ${Date.now() - startTime}ms}`
      );
    });
  }
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  dtsPlugin,
  getTSConfig,
  humanizeFileSize
});
//# sourceMappingURL=index.js.map